{%- from "linux/map.jinja" import system with context %}
{%- if system.enabled %}

{%- for file_name, file in system.file.items() %}

  {%- if file.decode is defined %}
    {%- if file.name is defined %}
      {%- set dirname = salt['file.dirname'](file.name) %}
    {%- else %}
      {%- set dirname = salt['file.dirname'](file_name) %}
    {%- endif %}
linux_directory_{{ dirname }}:
  file.directory:
    - name: {{ dirname }}
    - makedirs: {{ file.get('makedirs', 'True') }}
    - user: {{ file.get('user', 'root') }}
    - group: {{ file.get('group', 'root') }}

linux_file_{{ file_name }}:
  file.decode:
    {%- if file.name is defined %}
    - name: {{ file.name }}
    {%- else %}
    - name: {{ file_name }}
    {%- endif %}
    - encoding_type: base64
    {%- if file.contents_pillar is defined %}
    - contents_pillar: {{ file.contents_pillar }}
    {%- elif file.encoded_data is defined %}
    - encoded_data: |
        {{ file.encoded_data | indent(8) }}
    {%- endif %}
  {%- else %}
linux_file_{{ file_name }}:
    {%- if file.serialize is defined %}
  file.serialize:
    - formatter: {{ file.serialize }}
      {%- if file.contents is defined  %}
    - dataset: {{ file.contents|yaml }}
      {%- elif file.contents_pillar is defined %}
    - dataset_pillar: {{ file.contents_pillar }}
      {%- endif %}
    {%- else %}
  file.managed:
    {%- if file.secured_source is defined %}
      {%- set file_source = file.secured_source.get('protocol', 'http') + '://' +
                            file.secured_source.get('user') + ':' +
                            file.secured_source.get('password') + '@' +
                            file.secured_source.get('url') %}
    - source: {{ file_source }}
      {%- if file.secured_hash is defined %}
      {%- set file_hash = file.secured_hash.get('protocol', file.secured_source.get('protocol', 'http')) + '://' +
                          file.secured_hash.get('user', file.secured_source.get('user')) + ':' +
                          file.secured_hash.get('password', file.secured_source.get('password')) + '@' +
                          file.secured_hash.get('url', file.secured_source.get('url')) %}
    - source_hash: {{ file_hash }}
      {%- elif file.hash is defined %}
    - source_hash: {{ file.hash }}
      {%- else %}
    - skip_verify: True
      {%- endif %}
    {%- elif file.source is defined %}
    - source: {{ file.source }}
    {%- if file.hash is defined %}
    - source_hash: {{ file.hash }}
    {%- else %}
    - skip_verify: True
    {%- endif %}
    {%- if file.template is defined %}
    - template: {{ file.template }}
    {%- endif %}
    {%- elif file.contents is defined %}
    - contents: {{ file.contents|yaml }}
    {%- elif file.contents_pillar is defined %}
    - contents_pillar: {{ file.contents_pillar }}
    {%- elif file.contents_grains is defined %}
    - contents_grains: {{ file.contents_grains }}
    {%- endif %}
  {%- endif %}
    {%- if file.name is defined %}
    - name: {{ file.name }}
    {%- else %}
    - name: {{ file_name }}
    {%- endif %}
    - makedirs: {{ file.get('makedirs', 'True') }}
    - user: {{ file.get('user', 'root') }}
    - group: {{ file.get('group', 'root') }}
    {%- if file.mode is defined %}
    - mode: {{ file.mode }}
    {%- endif %}
    {%- if file.dir_mode is defined %}
    - dir_mode: {{ file.dir_mode }}
    {%- endif %}
    {%- if file.encoding is defined %}
    - encoding: {{ file.encoding }}
    {%- endif %}
{%- endif %}

{%- endfor %}

{%- endif %}
